#property copyright " Bernd Kreuss" 
#property indicator_chart_window 
#property indicator_buffers 1 
#property indicator_color1 Red
#property indicator_width1 2 
#include <mt4R.mqh>
// Eventhandler
#include <eventhandler.mqh>

extern string RTermPath_desc="Commandline to call RTERM.EXE";
extern string RTermPath="C:\Apps\R\R-2.15.3\bin\i386\Rterm.exe --no-save";
//
extern string RWorkPath_desc="Where are the R-Snapshots to be saved?";
extern string RWorkPath="C:/Apps/FXCM MetaTrader 4/experts/MT4-R/R-Auswertungen";
extern int order = 200;
extern string retro_desc="How many bars into the past to begin a prediction";
extern int retro=0;
//
#define PM_OPEN 0
#define PM_HIGH 1
#define PM_LOW 2
#define PM_CLOSE 3
#define PM_TYPICAL 4
#define PM_MEDIAN 5
#define PM_WEIGHTED 6
extern string priceModel_desc1="Open=0, High=1, Low=2, Close=3,";
extern string priceModel_desc2="Typical=4, Median=5, Weighted=6";
extern string priceModel_desc3="Default: 3 (Close)";
extern int priceModel=3;
//
extern string back_desc = "How many bars into the past from retro to sample the data";
extern int back = 2000;
extern string ahead_desc="How many bars to predict";
extern int ahead = 50;
extern bool saveWS=false;
extern string Rmethod_desc="Allowed: yule-walker, burg, ols, mle, yw";
extern string Rmethod="ols";
static int curPeriod=0;

int R;
double buf_prediction[];

int init(){
	SetIndexBuffer(0, buf_prediction);
	SetIndexStyle(0, DRAW_LINE);
	SetIndexShift(0, ahead-retro);
	curPeriod=Period();
	R = RInit(RTermPath, 2);
	RExecute(R, "setwd('" + RWorkPath + "')");
	Comment("$Rev: 453 $ Retrace " + retro + " bars, history: " + back + " bars, method: " + Rmethod + " , order: " + order + 
	", Datapath: " + RWorkPath);
	// Initialize Eventhandler to provide onBar and onTick Events.
	events.initEvents();
	doRDraw();
	if (saveWS) {
		RExecute(R,"save.image(file='"+ Symbol() + "_" + Period() + ".RData')");
	}
}

int deinit(){
	Comment("");
	RDeinit(R);
}

int start(){
	// Perform eventhandler calls.
	events.doEvents();
	return(0);   
}

void events.onBar(int pPeriod) {
	switch (pPeriod) {
	case PERIOD_M1:
		// Print("onBar: working on M1...");
		doRDraw();
		break;
	case PERIOD_M5:
		// Print("onBar: working on M5...");
		if (saveWS) {
			RExecute(R,"save.image(file='"+ Symbol() + "_" + Period() + ".RData')");
		}
		break;
	case PERIOD_M15:
		break;
	case PERIOD_M30:
		break;
	case PERIOD_H1:
		break;
	case PERIOD_H4:
		break;
	case PERIOD_D1:
		break;
	case PERIOD_W1:
		break;
	default:
		break;
	}
	return;
}

void events.onTick() {
	return;
}

void doRDraw() {
	static int curPeriod=0;
	// double hist[];
	double hist_o[];
	double hist_h[];
	double hist_l[];
	double hist_c[];
	double hist_m[];
	double hist_t[];
	double hist_w[];
	// Für die Zeitpunkte
	double hist_dt[];
	// Hier stehen die Vorhersagen...
	double pred[];
	int i;
	// Umschalten von Timeframes macht komische Sachen! :-)
	if (Period()!=curPeriod) {
		curPeriod=Period();
		doRDraw();
	}
	if (RIsBusy(R)){
		// last RExecuteAsync() is still not finished, do nothing. 
		return(0);
	}
	// make a (new) prediction
	// move some history over to R   
	ArrayResize(hist_o, back);
	ArrayResize(hist_h, back);
	ArrayResize(hist_l, back);
	ArrayResize(hist_c, back);
	ArrayResize(hist_m, back);
	ArrayResize(hist_t, back);
	ArrayResize(hist_w, back);
	ArrayResize(hist_dt, back);
	int j=0;
	for (i=back; i>=0; i--){
		j=back - (i - retro);
		hist_o[i] = Open[j];
		hist_h[i] = High[j];
		hist_l[i] = Low[j];
		hist_c[i] = Close[j];
		hist_m[i] = (High[j] + Low[j])/2;
		hist_t[i] = (High[j] + Low[j] + Close[j])/3;
		hist_w[i] = (High[j] + Low[j] + 2*Close[j])/4;
		hist_dt[i]= Time[j];
	}
	RAssignVector(R, "hist_o", 	hist_o, ArraySize(hist_o));
	RAssignVector(R, "hist_h", 	hist_h,	ArraySize(hist_h));
	RAssignVector(R, "hist_l", 	hist_l, ArraySize(hist_l));
	RAssignVector(R, "hist_c", 	hist_c, ArraySize(hist_c));
	RAssignVector(R, "hist_m", 	hist_m, ArraySize(hist_m));
	RAssignVector(R, "hist_t", 	hist_t, ArraySize(hist_t));
	RAssignVector(R, "hist_w", 	hist_w, ArraySize(hist_w));
	RAssignVector(R, "hist_dt", hist_dt, ArraySize(hist_dt));
	switch (priceModel) {
		case PM_OPEN:
			RExecute(R, "hist <- hist_o");
			break;
		case PM_HIGH:
			RExecute(R, "hist <- hist_h");
			break;
		case PM_LOW:
			RExecute(R, "hist <- hist_l");
			break;
		case PM_CLOSE:
			RExecute(R, "hist <- hist_c");
			break;
		case PM_TYPICAL:
			RExecute(R, "hist <- hist_t");
			break;
		case PM_MEDIAN:
			RExecute(R, "hist <- hist_m");
			break;
		case PM_WEIGHTED:
			RExecute(R, "hist <- hist_w");
			break;
		default:
			RExecute(R, "hist <- hist_c");
			break;
	}
   
	RAssignInteger(R, "ord", order);
	RAssignInteger(R, "ahead", ahead);
	string arArguments="model <- ar(hist, aic=FALSE, order=ord, method='" + Rmethod  + "')";
	RExecute(R, arArguments);
	// there exists a model (the variable is set). 
	// This means a previously started RExecuteAsync() has finished. 
	// we can now predict from this model and plot it.
	RExecute(R, "pred <- predict(model, n.ahead=ahead)$pred");
	ArrayResize(pred, ahead);
	RGetVector(R, "pred", pred, ahead);
	for (i=0; i<ahead; i++){
		buf_prediction[i] = pred[ahead - i - 1];
	}
}
